function [OutCellArray, Outconfig]=FreqMap_TurkeyFilter(InCellArray, Inconfig)
%FreqMap_TurkeyFilter applied a low pass Turkey filter to the map and it requires a
%input for the cutoff frequency and the ratio of the constant section
%to the filter size ( 0<=ratio<=1).  The filter tapers to 0.5 at the cutoff
%frequency.
%
%Inputs: Frequency Map
%Outputs: Frequency Map
%
%Author: Ping Zhou
%Email: 
%
% Version history:
% 9/10/10 - Version 1
% 8/20/11 - Version 1.1 Dae Wook Kim letter2dwk@hotmail.com (x label and
% title for the figure)
% 1/13/12 - Version 2.0 Michael Palmer mikeppalmer@gmail.com (added support
% for rectangular data matrix and plots in x and y).

OutCellArray{1}=InCellArray{1};
OutCellArray{1}.type='freqmap';

X_Max_Frequency=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Max Frequency'),2});
Y_Max_Frequency=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Max Frequency'),2});
% Compare xunit and yunit to make sure they are same.
xunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
xunit=strrep(xunit, '1/', '');
yunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
yunit=strrep(yunit, '1/', '');

% If xunit and yunit are not the same. Warn user and quit module.
if(xunit ~= yunit) 
    msgbox('Please check that xunit and yunit match in the header of the freqmap.','Invalid Units','warn','modal');
    return;
end

ptsX = size(InCellArray{1}.data, 2);
ptsY = size(InCellArray{1}.data, 1);

Outconfig=Inconfig;

validinputs=0;

f=Inconfig{strcmpi(Inconfig, 'f'),2};
alpha=Inconfig{strcmpi(Inconfig, 'alpha'),2};

%Get user input
while ~validinputs
    answers = inputdlg({['The cutoff frequency 1/' xunit ':'], 'The ratio of constant section to filter size:'},...
        'Input parameters', 1, {num2str(f), num2str(alpha)});
    if ~isempty(answers)
        tempf=str2num(answers{1});  
        tempalpha=str2num(answers{2});

        if ((~isempty(tempf))&& (~isempty(tempalpha)))
            validinputs=1;           
            f=tempf;
            alpha = tempalpha;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'f'),2}=num2str(f);
Outconfig{strcmpi(Outconfig, 'alpha'),2}=num2str(alpha);

x = linspace(-X_Max_Frequency, X_Max_Frequency, ptsX);
y = linspace(-Y_Max_Frequency, Y_Max_Frequency, ptsY);

[X, Y] = meshgrid(x, y);
R = sqrt(X.^2+Y.^2);

TurkeyFilter = 0.5*(1+cos(pi*(R-alpha*f)/(2*(1-alpha)*f)));
TurkeyFilter(R<f*alpha) = 1;
TurkeyFilter(R>(2-alpha)*f) = 0;
size(TurkeyFilter)
% Apply turkey filter in x and y
figure
subplot(2,1,1);
plot(x, TurkeyFilter(round(ptsY/2), :));
title('Applied Turkey Filter in X');
xlabel(['1/', xunit]);

subplot(2,1,2);
plot(y, TurkeyFilter(:, round(ptsX/2)));
title('Applied Turkey Filter in Y');
xlabel(['1/', yunit]);

data = InCellArray{1}.data;
data(isnan(data))=0;

OutCellArray{1}.data=data.*TurkeyFilter;