%% Display statistics for map datasets
function [OutConfig] = MapStatistics(InCellArray, InConfig)
% This module displays a set of statistics about a Map.  Specifically, the
% slopes (or gradients or derivatives) are computed and plotted together
% with the surface.  The minimum, maximum, standard deviation, RMS and
% other similar values of the surface and slopes are also provided.
%
% Inputs: Map
% Outputs: None
%
% Author: Benjamin Lewis
% Email: blewis@email.arizona.edu
%
% Version History:
% 2010/11/30 BL  - Version 2
% 2012/02/08 ARL - Updated for SAGUARO Release 1.3

    % Constants
    SAGUAROcolor=[214/255,216/255,192/255];
    kShowSlopeXY  = 1;
    kShowSlopeRT  = 2;
    kShowSlopeMag = 3;
    kDefaultOutlierFactor = 3.0;

    % Variables used in this function and in nested functions
    filtSurface = [];
    slopeMag = [];
    slopeX = [];
    slopeY = [];
    slopeRad = [];
    slopeTan = [];
    slopeMagUnits = [];
    slopeXUnits = [];
    slopeYUnits = [];
    slopeRadUnits = [];
    slopeTanUnits = [];
    surfaceRms = [];
    surfaceStd = [];
    slopeMagRms = [];
    slopeXRms = [];
    slopeYRms = [];
    slopeRadRms = [];
    slopeTanRms = [];
    MagOnlyValues = [];
    XYValues = [];
    RTValues = [];
    cbarSlopeXR = [];
    cbarSlopeYT = [];


    % Copy user input configuration parameters to output
    OutConfig = InConfig;

    % Only handle Map datasets
    if ~strcmpi(InCellArray{1}.type, 'map')
        % Dataset type is not supported
        return;    
    end

    % Read user configuration parameters
    configShowSlopes      = uint8(str2double(OutConfig{strcmpi(OutConfig, 'Show Slopes'),2}));
    configDiscardOutliers = logical(str2double(OutConfig{strcmpi(OutConfig, 'Discard Outliers'),2}));
    configOutlierFactor   = abs(str2double(OutConfig{strcmpi(OutConfig, 'Outlier Factor'),2}));

    % Check user configuration parameters are valid
    if configShowSlopes < kShowSlopeXY
        configShowSlopes = kShowSlopeXY;
    elseif configShowSlopes > kShowSlopeMag
        configShowSlopes = kShowSlopeMag;
    end
    if isnan(configOutlierFactor)
        configOutlierFactor = kDefaultOutlierFactor;
    end

    % Read parameters from header
    SizeX  = str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X size'),2});
    SizeY  = str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y size'),2});
    UnitsX = InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X unit'),2};
    UnitsY = InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y unit'),2};
    UnitsZ = InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Z unit'),2};

    % Check units
    bXYUnitsMatch  = strcmp(UnitsX, UnitsY);
    bZXUnitsMatch  = strcmp(UnitsZ, UnitsX);
    bZYUnitsMatch  = strcmp(UnitsZ, UnitsY);
    bAllUnitsMatch = bZXUnitsMatch & bZYUnitsMatch;

    % Read map from data block
    rawSurface = real(InCellArray{1}.data);

    % If map doesn't contain any data, set it to all zeros
    if ~any(~isnan(rawSurface(:)))
        rawSurface = zeros(size(rawSurface));
    end

    % Get size of map in each direction
    [ptsY, ptsX] = size(rawSurface);

    % Set center of map
    centerX = (ptsX - 1) / 2;
    centerY = (ptsY - 1) / 2;

    % Compute the mean and standard deviation of the raw map data
    % Note: the sample standard deviation is calculated rather than the
    % population standard deviation (std(..., 1)) since the map typically
    % represents discrete sample positions over a continuous surface.
    RawMean   = mean(rawSurface(~isnan(rawSurface)));
    RawStdDev = std(rawSurface(~isnan(rawSurface)));

    % Initialize the GUI
    % Position and size of figure
    kFigLeft       = 200;
    kFigBot        = 200;
    kFigWidth      = 940;
    kFigHeight     = 540;

    % Positions and sizes of plots
    kPlotTop       = 310;
    kPlotBot       =  50;
    kPlotLeft      =  35;
    kPlotRight     = 370;
    kPlotWidth     = 245;
    kPlotHeight    = 190;
    kPlotTopLeft   = [kPlotLeft,  kPlotTop];
    kPlotTopRight  = [kPlotRight, kPlotTop];
    kPlotBotLeft   = [kPlotLeft,  kPlotBot];
    kPlotBotRight  = [kPlotRight, kPlotBot];
    kPlotSize      = [kPlotWidth,  kPlotHeight];

    % Positions and sizes of controls
    kTextHeight    = 15;
    kControlsLeft  = 688;
    kControlsWidth = 242;
    kControlsHSep  = 3;
    kControlsVSep  = 20;
    kTableBot      = 225;

    % Create the figure
    % Note: if figure visibility is initially set to off, then the plots
    % are drawn slightly off-center from the axis grids, which looks bad.
    fig = figure( ...
        'Tag', 'figure', ...
        'Visible', 'on', ...
        'Position', [kFigLeft, kFigBot, kFigWidth, kFigHeight], ...
        'Color', SAGUAROcolor, ...
        'Toolbar', 'figure', ...
        'Name', 'SAGUARO Statistics', ...
        'Resize', 'off');
    movegui(fig, 'center');

    % Draw the axes to hold the four plots
    axSurface  = axes('Units', 'Pixels', 'Position', [kPlotBotLeft,  kPlotSize]);
    axSlopeMag = axes('Units', 'Pixels', 'Position', [kPlotBotRight, kPlotSize]);
    axSlopeXR  = axes('Units', 'Pixels', 'Position', [kPlotTopLeft,  kPlotSize]);
    axSlopeYT  = axes('Units', 'Pixels', 'Position', [kPlotTopRight, kPlotSize]);

    % Draw text boxes to indicate that the plots are being calculated
    kWaitSettings = ...
        { ...
            'Style', 'text', ...
            'BackgroundColor', get(fig, 'Color'), ...
            'String', 'Calculating...', ...
            'HorizontalAlignment', 'center' ...
        };
    txtWait = ...
        [ ...
            uicontrol(kWaitSettings{:}, 'Position', [kPlotBotLeft  + kPlotSize .* [0.0 0.5], kPlotWidth, kTextHeight]), ...
            uicontrol(kWaitSettings{:}, 'Position', [kPlotBotRight + kPlotSize .* [0.0 0.5], kPlotWidth, kTextHeight]), ...
            uicontrol(kWaitSettings{:}, 'Position', [kPlotTopLeft  + kPlotSize .* [0.0 0.5], kPlotWidth, kTextHeight]), ...
            uicontrol(kWaitSettings{:}, 'Position', [kPlotTopRight + kPlotSize .* [0.0 0.5], kPlotWidth, kTextHeight]) ...
        ];

    % Set up a table displaying statistical values
    vertPos = kTableBot;
    ValueTable = uitable( ...
        'Tag', 'tblValues', ...
        'Units', 'Pixels', ...
        'Position', [kControlsLeft, vertPos, kControlsWidth, 295], ...
        'RowName', [], ...
        'ColumnWidth', {80}, ...
        'ColumnName', {'Measure', 'Value', 'Units'});

    % Set up controls for user-settable options
    % Radio buttons to select which slope plots and data to display
    vertPos = vertPos - kControlsVSep * 5;
    bgShowSlopes = uibuttongroup( ...
        'Tag', 'bgShowSlopes', ...
        'BackgroundColor', get(fig, 'Color'), ...
        'BorderType', 'none', ...
        'Units', 'Pixels', ...
        'Position', [kControlsLeft, vertPos, kControlsWidth, kControlsVSep * 4]);

    btnsShowSlope = ...
        [ ...
            % X/Y Slopes
            uicontrol( ...
                'Tag', 'rbShowSlopeXY', ...
                'Style', 'Radio', ...
                'BackgroundColor', get(fig, 'Color'), ...
                'String', 'Show Slope X and Slope Y', ...
                'Units', 'Pixels', ...
                'Position', [0, kControlsVSep * 3, kControlsWidth, kTextHeight], ...
                'UserData', kShowSlopeXY, ...
                'Parent', bgShowSlopes);

            % Radial/Tangential Slopes
            uicontrol( ...
                'Tag', 'rbShowSlopeRT', ...
                'Style', 'Radio', ...
                'BackgroundColor', get(fig, 'Color'), ...
                'String', 'Show Radial and Tangential Slope', ...
                'Units', 'Pixels', ...
                'Position', [0, kControlsVSep * 2, kControlsWidth, kTextHeight], ...
                'UserData', kShowSlopeRT, ...
                'Parent', bgShowSlopes);

            % Magnitude Slope only
            uicontrol( ...
                'Tag', 'rbShowSlopeMag', ...
                'Style', 'Radio', ...
                'BackgroundColor', get(fig, 'Color'), ...
                'String', 'Show Slope Magnitude only', ...
                'Units', 'Pixels', ...
                'Position', [0, kControlsVSep, kControlsWidth, kTextHeight], ...
                'UserData', kShowSlopeMag, ...
                'Parent', bgShowSlopes);
        ];

    set(bgShowSlopes, 'SelectionChangeFcn', @showSlopes_Callback);
    % Select the radio button corresponding to the current configuration
    btnValues = get(btnsShowSlope, 'UserData');
    set(bgShowSlopes, 'SelectedObject', ...
        btnsShowSlope([btnValues{:}] == configShowSlopes));

    % Controls to discard N-sigma outliers from map
    %   Checkbox to enable/disable
    vertPos = vertPos - kControlsVSep;
    uicontrol( ...
        'Tag', 'cbDiscardOutliers', ...
        'Style', 'checkbox', ...
        'BackgroundColor', get(fig, 'Color'), ...
        'String', 'Discard outliers from map', ...
        'Units', 'Pixels', ...
        'Position', [kControlsLeft, vertPos, kControlsWidth, kTextHeight], ...
        'Max', true, ...
        'Min', false, ...
        'Value', logical(configDiscardOutliers), ...
        'Callback', @discardOutliers_Callback);

    %   Leading text label
    vertPos  = vertPos - kControlsVSep;
    horizPos = kControlsLeft + 15;
    controlWidth = 35;
    uicontrol( ...
        'Style', 'text', ...
        'BackgroundColor', get(fig, 'Color'), ...
        'String', 'Limit = ', ...
        'HorizontalAlignment', 'right', ...
        'Position', [horizPos, vertPos, controlWidth, kTextHeight]);

    %   Edit box for N value
    horizPos = horizPos + controlWidth + kControlsHSep;
    controlWidth = 40;
    uicontrol( ...
        'Style', 'edit', ...
        'String', num2str(configOutlierFactor), ...
        'HorizontalAlignment', 'right', ...
        'Position', [horizPos, vertPos, controlWidth, kTextHeight], ...
        'Callback', @outlierFactor_Callback);

    %   Trailing text label
    horizPos = horizPos + controlWidth + kControlsHSep;
    controlWidth = 50;
    uicontrol( ...
        'Style', 'text', ...
        'BackgroundColor', get(fig, 'Color'), ...
        'String', ' std.dev.', ...
        'HorizontalAlignment', 'left', ...
        'Position', [horizPos, vertPos, controlWidth, kTextHeight]);


    % Calculate and plot data
    calculateData();
    displayData();

    % Select the figure
    figure(fig);

    % If not running in automated mode
    if ~any(strcmpi(OutConfig(:,1), 'AUTOMATED MODE'))
        % Wait for user to close figure before returning.
        % Note: this interferes with automated testing, hence the check
        % to ensure the module isn't being run in automated mode.
        uiwait(fig);
    end

    % Update user configuration parameters
    OutConfig{strcmpi(OutConfig, 'Show Slopes'),2} = num2str(configShowSlopes);
    OutConfig{strcmpi(OutConfig, 'Discard Outliers'),2} = num2str(configDiscardOutliers);
    OutConfig{strcmpi(OutConfig, 'Outlier Factor'),2} = num2str(configOutlierFactor);


    %% Calculate the data to be displayed
    function calculateData()
        % Display the wait text boxes
        set(txtWait, 'Visible', 'on');

        % Remove outliers from raw surface to generate filtered surface
        filtSurface = rawSurface;
        if configDiscardOutliers
            filtSurface(filtSurface > RawMean + configOutlierFactor * RawStdDev) = NaN;
            filtSurface(filtSurface < RawMean - configOutlierFactor * RawStdDev) = NaN;
        end

        % Calculate slopes using normalized sample spacing
        % Note: gradient only calculates a single output if the input array
        %       is a vector
        if (ptsX > 1 && ptsY > 1)
            [slopeX, slopeY] = gradient(filtSurface, SizeX / ptsX, SizeY / ptsY);
        elseif (ptsX > 1)
            slopeX = gradient(filtSurface, SizeX / ptsX);
            slopeY = zeros(size(filtSurface));
        elseif (ptsY > 1)
            slopeX = zeros(size(filtSurface));
            slopeY = gradient(filtSurface, SizeY / ptsY);
        else
            slopeX = zeros(size(filtSurface));
            slopeY = zeros(size(filtSurface));
        end
        slopeXUnits = [UnitsZ '/' UnitsX];
        slopeYUnits = [UnitsZ '/' UnitsY];

        % Calculate slope magnitude, radial slope, and tangential slope if
        % X and Y units match
        if (bXYUnitsMatch)
            slopeMag = sqrt(slopeX.^2 + slopeY.^2);
            slopeMagUnits = [UnitsZ '/' UnitsX];

            % Calculate the distance of each pixel from the center of the map
            [slopeDistX,slopeDistY] = meshgrid((0:(ptsX-1))-centerX, ...
                                               (0:(ptsY-1))-centerY);

            % Calculate the angle of each pixel with respect to the center of the map
            pixelAngle = cart2pol(slopeDistX,slopeDistY);

            % Rotate the X and Y slopes by the pixel angle to get the radial and tangential slopes
            [slopeRad, slopeTan] = cartRotate(slopeX, slopeY, pixelAngle);
            slopeRadUnits = slopeMagUnits;
            slopeTanUnits = slopeMagUnits;
        else
            % These slopes can't be calculated if X and Y units don't match
            slopeMag = NaN(size(filtSurface));
            slopeRad = slopeMag;
            slopeTan = slopeMag;
            slopeMagUnits = 'N/A';
            slopeRadUnits = slopeMagUnits;
            slopeTanUnits = slopeMagUnits;
        end

        % Convert slopes to microradians if corresponding units match
        if (bAllUnitsMatch)
            slopeMag = 1e6 .* atan(slopeMag);
            slopeRad = 1e6 .* atan(slopeRad);
            slopeTan = 1e6 .* atan(slopeTan);
            slopeMagUnits = 'rad';
            slopeRadUnits = slopeMagUnits;
            slopeTanUnits = slopeMagUnits;
        end
        if (bZXUnitsMatch)
            slopeX = 1e6 .* atan(slopeX);
            slopeXUnits = 'rad';
        end
        if (bZYUnitsMatch)
            slopeY = 1e6 .* atan(slopeY);
            slopeYUnits = 'rad';
        end

        % Compute summary values
        surfaceRms  = rms(filtSurface);
        surfaceStd  = std(filtSurface(~isnan(filtSurface)));
        slopeMagRms = rms(slopeMag);
        slopeXRms   = rms(slopeX);
        slopeYRms   = rms(slopeY);
        slopeRadRms = rms(slopeRad);
        slopeTanRms = rms(slopeTan);

        % Set up arrays of values for table
        MagOnlyValues = ...
            { ...
                'Surface RMS', surfaceRms, UnitsZ; ...
                'Surface StdDev', surfaceStd, UnitsZ; ...
                'Surface P-V', max(filtSurface(~isnan(filtSurface)))-min(filtSurface(~isnan(filtSurface))), UnitsZ; ...
                ' ', ' ', ' '; ...
                'Surface Min', min(filtSurface(~isnan(filtSurface))), UnitsZ; ...
                'Surface Max', max(filtSurface(~isnan(filtSurface))), UnitsZ; ...
                'Slope Mag RMS', slopeMagRms, slopeMagUnits; ...
                'Slope Mag Min', min(slopeMag(~isnan(slopeMag))), slopeMagUnits; ...
                'Slope Mag Max', max(slopeMag(~isnan(slopeMag))), slopeMagUnits; ...
             };
        XYValues = vertcat( MagOnlyValues, ...
            { ...
                'Slope X RMS', slopeXRms, slopeXUnits; ...
                'Slope X Min', min(slopeX(~isnan(slopeX))), slopeXUnits; ...
                'Slope X Max', max(slopeX(~isnan(slopeX))), slopeXUnits; ...
                'Slope Y RMS', slopeYRms, slopeYUnits; ...
                'Slope Y Min', min(slopeY(~isnan(slopeY))), slopeYUnits; ...
                'Slope Y Max', max(slopeY(~isnan(slopeY))), slopeYUnits; ...
             });
        RTValues = vertcat( MagOnlyValues, ...
            { ...
                'Slope Rad RMS', slopeRadRms, slopeRadUnits; ...
                'Slope Rad Min', min(slopeRad(~isnan(slopeRad))), slopeRadUnits; ...
                'Slope Rad Max', max(slopeRad(~isnan(slopeRad))), slopeRadUnits; ...
                'Slope Tan RMS', slopeTanRms, slopeTanUnits; ...
                'Slope Tan Min', min(slopeTan(~isnan(slopeTan))), slopeTanUnits; ...
                'Slope Tan Max', max(slopeTan(~isnan(slopeTan))), slopeTanUnits; ...
             });
    end


    %% Display data in plots and table
    function displayData()
        % Draw plots
        drawMap(axSurface, filtSurface, ['Surface Map (' UnitsZ ')']);
        drawMap(axSlopeMag, slopeMag, ['Slope Magnitude Map (' slopeMagUnits ')']);
        if (configShowSlopes == kShowSlopeXY)
            cbarSlopeXR = drawMap(axSlopeXR, slopeX, ['Slope X Map (' slopeXUnits ')']);
            cbarSlopeYT = drawMap(axSlopeYT, slopeY, ['Slope Y Map (' slopeYUnits ')']);
        else
            cbarSlopeXR = drawMap(axSlopeXR, slopeRad, ['Slope Radial Map (' slopeRadUnits ')']);
            cbarSlopeYT = drawMap(axSlopeYT, slopeTan, ['Slope Tangential Map (' slopeTanUnits ')']);
        end

        % Reset the positions of the plots (they appear to move after
        % being drawn)
        set(axSurface,  'Position', [kPlotBotLeft,  kPlotSize]);
        set(axSlopeMag, 'Position', [kPlotBotRight, kPlotSize]);
        set(axSlopeXR,  'Position', [kPlotTopLeft,  kPlotSize]);
        set(axSlopeYT,  'Position', [kPlotTopRight, kPlotSize]);

        % Set up the display according to the configured settings
        showHideSlopeData();

        % Hide the wait text boxes
        set(txtWait, 'Visible', 'off');
    end


    %% Plot a single map
    function cbarPlot = drawMap(axPlot, data, txTitle)
        % Clear the plot
        reset(axPlot);

        % Draw the data as a square, scaled image
        imagesc(data,'Parent',axPlot,'AlphaData',~isnan(data));
        axis(axPlot, 'square');

        % Set the axis ticks
        if (ptsX > 1)
            set(axPlot, ...
                'XTickLabel', (-0.5:0.25:0.5) * SizeX, ...
            	'XTick', (0:.25:1) * ptsX);
        else
            set(axPlot, ...
                'XTickLabel', 0, ...
            	'XTick', 0);
        end
        if (ptsY > 1)
            set(axPlot, ...
                'YTick', (0:.25:1) * ptsY, ...
            	'YTickLabel', (-0.5:0.25:0.5) * SizeY);
        else
            set(axPlot, ...
                'YTickLabel', 0, ...
            	'YTick', 0);
        end

        % Label the axes with the units
        xlabel(axPlot, UnitsX);
        ylabel(axPlot, UnitsY);

        % Set the plot title
        title(axPlot, txTitle);

        % Draw the colorbar
        cbarPlot = colorbar('peer', axPlot);
    end


    %% Show or hide Slope X/Y or Slope Radial/Tangential data
    function showHideSlopeData()
        if (configShowSlopes == kShowSlopeMag)
            % Hide slope X/Y or Radial/Tangential plots and data
            set(axSlopeXR, 'Visible', 'off');
            set(get(axSlopeXR, 'Children'), 'Visible', 'off');
            set(cbarSlopeXR, 'Visible', 'off');
            set(axSlopeYT, 'Visible', 'off');
            set(get(axSlopeYT, 'Children'), 'Visible', 'off');
            set(cbarSlopeYT, 'Visible', 'off');
            set(ValueTable, 'Data', MagOnlyValues);

            % Move the standard plots up
            set(axSurface,  'Position', [kPlotTopLeft,  kPlotSize]);
            set(axSlopeMag, 'Position', [kPlotTopRight, kPlotSize]);
        else
            % Move the standard plots down
            set(axSurface,  'Position', [kPlotBotLeft,  kPlotSize]);
            set(axSlopeMag, 'Position', [kPlotBotRight, kPlotSize]);

            % Display slope X/Y or Radial/Tangential plots and data
            set(axSlopeXR, 'Visible', 'on');
            set(get(axSlopeXR, 'Children'), 'Visible', 'on');
            set(cbarSlopeXR, 'Visible', 'on');
            set(axSlopeYT, 'Visible', 'on');
            set(get(axSlopeYT, 'Children'), 'Visible', 'on');
            set(cbarSlopeYT, 'Visible', 'on');
            if (configShowSlopes == kShowSlopeXY)
                set(ValueTable, 'Data', XYValues);
            else
                set(ValueTable, 'Data', RTValues);
            end
        end
    end


    %% A Show Slope radio button has been changed
    function showSlopes_Callback(hObject, ~)
        % Set which slope plots and data to display based on which radio
        % button is selected
        configShowSlopes = get(get(hObject, 'SelectedObject'), 'UserData');

        % Plot data
        displayData();
    end


    %% The Discard Outliers checkbox setting has been changed
    function discardOutliers_Callback(hObject, ~)
        % Discard or include outliers from surface data depending on
        % whether the checkbox is checked or not
        configDiscardOutliers = (get(hObject,'Value') == get(hObject,'Max'));

        % Recalculate and plot data
        calculateData();
        displayData();
    end


    %% The Outlier Factor edit box contents have been changed
    function outlierFactor_Callback(hObject, ~)
        % Discard or include outliers from surface data depending on
        % whether the checkbox is checked or not
        userValue = str2double(get(hObject,'String'));
        if isnan(userValue)
            errordlg('Outlier limit must be a numeric value', ...
                     'Invalid Input', 'modal');
            uicontrol(hObject);
            return
        end
        configOutlierFactor = abs(userValue);
        if configDiscardOutliers
            % Recalculate and plot data
            calculateData();
            displayData();
        end
    end


    %% Calculate the root-mean-square of an array
    function rmsOut = rms(inArray)
        % Check that the input array contains some valid data
        if any(~isnan(inArray(:)))
            % Ignore any NaNs in the input array
            rmsOut = sqrt(mean(inArray(~isnan(inArray)).^2));
        else
            % No data in the input array: return an empty array
            rmsOut = [];
        end
    end


    %% Rotate Cartesian coordinates clockwise by an angle
    function [xOut, yOut] = cartRotate(xIn, yIn, theta)
        xOut =  xIn .* cos(theta) + yIn .* sin(theta);
        yOut = -xIn .* sin(theta) + yIn .* cos(theta);
    end
end
